package ca.gerrymatte.webutils.mailserver;

import java.util.*;
// import java.util.logging.*;
import org.jboss.logging.Logger;
import java.sql.*;

import javax.sql.*;
import javax.mail.*;

import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;
/**
 * Class <code>MySessionBean</code> provides a session scoped utility container and methods 
 * @author Gerry Matte
 *
 */
public class MySessionBean implements HttpSessionBindingListener,java.io.Serializable {

	/**
	 * MySessionBean provides a container for session context storage as well as utility methods 
	 * for the application and JDBC I/O methods.
	 * This module relies on the DataSource MAIL_DATASOURCE previously stored as a context attribute.
	 */
	private static final long serialVersionUID = 743447081092291355L;
	private Logger logger=Logger.getLogger(this.getClass().getName());
	/** the database connection for use during this browser session. **/
	private Connection c;
	/** the utility object for returning a Map of Post Offices**/
	private HashMap<Integer,String> postOffices=new HashMap<Integer,String>();
	/** the default selected postOffice has Id=1 **/
	private int selectedPostOfficeId=-1;
	private String selectedPostOffice;
	
	public MySessionBean() {
		logger.info("No argument constructor is called");
		DataSource ds = ca.gerrymatte.webutils.mailserver.DataSourceServlet.MAIL_DATASOURCE;
		try {
			c=ds.getConnection();
			setSelectedPostOfficeId(1);
			logger.info("A JDBC Connection was obtained and default PostOffice selected");
		} catch (SQLException e) {
			logger.error("A JDBC Connection was NOT obtained",e);
		}
	}
	
	public Map getPostOffices(){
		postOffices.clear();
		try {
			ResultSet rs = c.createStatement()
				.executeQuery("SELECT Id, Name FROM postoffice ORDER BY Name");
		      if(rs!=null) {
		    	  while (rs.next()) {
		    		  postOffices.put(rs.getInt("Id"),rs.getString("Name"));
		    	  }
		    	  rs.close();
		      }
		} catch (SQLException e) {
			logger.error("Unable to fetch from table postoffice.",e);
			System.err.println("Unable to fetch from table postoffice.");
			}
		return postOffices;
	}
	public void setPostOffices(HashMap<Integer,String> postOffices) {
		logger.error("Unable to modify table postoffice.");
	}
	public int getSelectedPostOfficeId() {
		if(c==null) logger.error("getSelectedPostofficeId(): connection is null");
		if(selectedPostOfficeId==-1) logger.error("getSelectedPostofficeId(): selectedPostofficeId is -1");
		return selectedPostOfficeId;
	}
	public void setSelectedPostOfficeId(int selectedPostOfficeId) {
		this.selectedPostOfficeId = selectedPostOfficeId;
		if(postOffices.isEmpty()) getPostOffices();
		if(postOffices.containsKey(selectedPostOfficeId)) {
			setSelectedPostOffice((String) postOffices.get(selectedPostOfficeId));
			}
		}
	public String getSelectedPostOffice() {
		if(c==null) logger.error("getSelectedPostoffice(): connection is null");
		if(selectedPostOffice==null) logger.error("getSelectedPostoffice(): selectedPostoffice is null");
		return selectedPostOffice;
	}
	/** set the String value of the selected post office and indirectly set the 
	 * integer Id of the selected post office by looking up the id using the String value. 
	 * Setting the String value of SelectedPostOffice does not update the database. 
	 * 
	 * @param selectedPostOffice (String)
	 */
	public void setSelectedPostOffice(String selectedPostOffice) {
		if(postOffices.containsValue(selectedPostOffice)) {
			this.selectedPostOffice = selectedPostOffice;
			for (Iterator it = postOffices.entrySet().iterator(); it.hasNext();) {
				   Map.Entry entry = (Map.Entry) it.next();
				   Integer key = (Integer) entry.getKey();
				   String value = (String) entry.getValue();
				   if(value.equalsIgnoreCase(selectedPostOffice)) {
					   // setSelectedPostOfficeId(key);
					   this.selectedPostOfficeId = key;
				   }
				}
			}	
		
	}

	@Override
	public void valueBound(HttpSessionBindingEvent arg0) {
		logger.info("valueBound to session");
	}

	@Override
	public void valueUnbound(HttpSessionBindingEvent arg0) {
		logger.info("valueUnbound to session");
		try {
			c.close();
			logger.info("Database connection closed");
		} catch (SQLException e) {
			logger.warn("Database connection error when closing",e);
		}
	}
}
